@@ -1,8 +1,4 @@
 package MyDatabase::Main;
-
-use warnings;
-use strict;
-
 use base qw/DBIx::Class::Schema/;
 __PACKAGE__->load_namespaces;
 
