@@ -1,4 +1,5 @@
-package DBIx::Class::PK::Auto::Pg;
+package # hide package from pause
+  DBIx::Class::PK::Auto::Pg;
 
 use strict;
 use warnings;
