@@ -16,7 +16,7 @@ DBIx::Class::ResultSource::View - ResultSource object representing a view
 
 =head1 SYNOPSIS
 
-  package MyDB::Schema::Result::Year2000CDs;
+  package MyApp::Schema::Result::Year2000CDs;
 
   use base qw/DBIx::Class::Core/;
 
@@ -63,7 +63,7 @@ case replaces the view name in a FROM clause in a subselect.
 
 =head1 EXAMPLES
 
-Having created the MyDB::Schema::Year2000CDs schema as shown in the SYNOPSIS
+Having created the MyApp::Schema::Year2000CDs schema as shown in the SYNOPSIS
 above, you can then:
 
   $2000_cds = $schema->resultset('Year2000CDs')
@@ -106,7 +106,7 @@ You could now say:
 
   $schema->resultset('Year2000CDs')->all();
 
-  SELECT cdid, artist, title FROM 
+  SELECT cdid, artist, title FROM
     (SELECT cdid, artist, title FROM cd WHERE year ='2000') me
 
 =back
@@ -129,10 +129,10 @@ database-based view.
 An SQL query for your view. Will not be translated across database
 syntaxes.
 
-=head2 deploy_depends_on 
+=head2 deploy_depends_on
 
   __PACKAGE__->result_source_instance->deploy_depends_on(
-      ["MyDB::Schema::Result::Year","MyDB::Schema::Result::CD"]
+      ["MyApp::Schema::Result::Year","MyApp::Schema::Result::CD"]
       );
 
 Specify the views (and only the views) that this view depends on.
