@@ -3,14 +3,18 @@ package DBIx::Class::Storage::DBI::MSSQL;
 use strict;
 use warnings;
 
-use base qw/DBIx::Class::Storage::DBI::UniqueIdentifier/;
+use base qw/
+  DBIx::Class::Storage::DBI::UniqueIdentifier
+  DBIx::Class::Storage::DBI::IdentityInsert
+/;
 use mro 'c3';
+
 use Try::Tiny;
 use List::Util 'first';
 use namespace::clean;
 
 __PACKAGE__->mk_group_accessors(simple => qw/
-  _identity _identity_method _pre_insert_sql _post_insert_sql
+  _identity _identity_method _no_scope_identity_query
 /);
 
 __PACKAGE__->sql_maker_class('DBIx::Class::SQLMaker::MSSQL');
@@ -21,54 +25,8 @@ __PACKAGE__->datetime_parser_type (
   'DBIx::Class::Storage::DBI::MSSQL::DateTime::Format'
 );
 
-
 __PACKAGE__->new_guid('NEWID()');
 
-sub _set_identity_insert {
-  my ($self, $table) = @_;
-
-  my $stmt = 'SET IDENTITY_INSERT %s %s';
-  $table   = $self->sql_maker->_quote($table);
-
-  $self->_pre_insert_sql (sprintf $stmt, $table, 'ON');
-  $self->_post_insert_sql(sprintf $stmt, $table, 'OFF');
-}
-
-sub insert_bulk {
-  my $self = shift;
-  my ($source, $cols, $data) = @_;
-
-  my $is_identity_insert =
-    (first { $_->{is_auto_increment} } values %{ $source->columns_info($cols) } )
-      ? 1
-      : 0
-  ;
-
-  if ($is_identity_insert) {
-     $self->_set_identity_insert ($source->name);
-  }
-
-  $self->next::method(@_);
-}
-
-sub insert {
-  my $self = shift;
-  my ($source, $to_insert) = @_;
-
-  my $supplied_col_info = $self->_resolve_column_info($source, [keys %$to_insert] );
-
-  my $is_identity_insert =
-    (first { $_->{is_auto_increment} } values %$supplied_col_info) ? 1 : 0;
-
-  if ($is_identity_insert) {
-     $self->_set_identity_insert ($source->name);
-  }
-
-  my $updated_cols = $self->next::method(@_);
-
-  return $updated_cols;
-}
-
 sub _prep_for_execute {
   my $self = shift;
   my ($op, $ident, $args) = @_;
@@ -94,15 +52,15 @@ sub _prep_for_execute {
 
   my ($sql, $bind) = $self->next::method (@_);
 
-  if ($op eq 'insert') {
-    if (my $prepend = $self->_pre_insert_sql) {
-      $sql = "${prepend}\n${sql}";
-      $self->_pre_insert_sql(undef);
-    }
-    if (my $append  = $self->_post_insert_sql) {
-      $sql = "${sql}\n${append}";
-      $self->_post_insert_sql(undef);
-    }
+  # SELECT SCOPE_IDENTITY only works within a statement scope. We
+  # must try to always use this particular idiom frist, as it is the
+  # only one that guarantees retrieving the correct id under high
+  # concurrency. When this fails we will fall back to whatever secondary
+  # retrieval method is specified in _identity_method, but at this
+  # point we don't have many guarantees we will get what we expected.
+  # http://msdn.microsoft.com/en-us/library/ms190315.aspx
+  # http://davidhayden.com/blog/dave/archive/2006/01/17/2736.aspx
+  if ($self->_perform_autoinc_retrieval and not $self->_no_scope_identity_query) {
     $sql .= "\nSELECT SCOPE_IDENTITY()";
   }
 
@@ -113,13 +71,20 @@ sub _execute {
   my $self = shift;
   my ($op) = @_;
 
+  # always list ctx - we need the $sth
   my ($rv, $sth, @bind) = $self->next::method(@_);
 
-  if ($op eq 'insert') {
+  if ($self->_perform_autoinc_retrieval) {
 
-    # this should bring back the result of SELECT SCOPE_IDENTITY() we tacked
+    # attempt to bring back the result of SELECT SCOPE_IDENTITY() we tacked
     # on in _prep_for_execute above
-    my ($identity) = try { $sth->fetchrow_array };
+    my $identity;
+
+    # we didn't even try on ftds
+    unless ($self->_no_scope_identity_query) {
+      ($identity) = try { $sth->fetchrow_array };
+      $sth->finish;
+    }
 
     # SCOPE_IDENTITY failed, but we can do something else
     if ( (! $identity) && $self->_identity_method) {
@@ -129,7 +94,6 @@ sub _execute {
     }
 
     $self->_identity($identity);
-    $sth->finish;
   }
 
   return wantarray ? ($rv, $sth, @bind) : $rv;
