@@ -2,11 +2,14 @@ package DBIx::Class::Storage::DBI::ODBC::Microsoft_SQL_Server;
 use strict;
 use warnings;
 
-use base qw/DBIx::Class::Storage::DBI::MSSQL/;
+use base qw/
+  DBIx::Class::Storage::DBI::ODBC
+  DBIx::Class::Storage::DBI::MSSQL
+/;
 use mro 'c3';
 use Scalar::Util 'reftype';
 use Try::Tiny;
-use Carp::Clan qw/^DBIx::Class/;
+use DBIx::Class::Carp;
 use namespace::clean;
 
 __PACKAGE__->mk_group_accessors(simple => qw/
@@ -33,12 +36,13 @@ L<DBIx::Class::Storage::DBI::MSSQL>.
 
   sudo aptitude install tdsodbc libdbd-odbc-perl unixodbc
 
-In case it is not already there put the following in C</etc/odbcinst.ini>:
+In case it is not already there put the following (adjust for non-64bit arch) in
+C</etc/odbcinst.ini>:
 
   [FreeTDS]
   Description = FreeTDS
-  Driver      = /usr/lib/odbc/libtdsodbc.so
-  Setup       = /usr/lib/odbc/libtdsS.so
+  Driver      = /usr/lib/x86_64-linux-gnu/odbc/libtdsodbc.so
+  Setup       = /usr/lib/x86_64-linux-gnu/odbc/libtdsS.so
   UsageCount  = 1
 
 Set your C<$dsn> in L<connect_info|DBIx::Class::Storage::DBI/connect_info> as follows:
@@ -75,15 +79,19 @@ The following options are alternative ways to enable concurrent executing
 statement support. Each has its own advantages and drawbacks and works on
 different platforms. Read each section carefully.
 
+For more details about using MAS in MSSQL over DBD::ODBC see this excellent
+document provided by EasySoft:
+L<http://www.easysoft.com/developer/languages/perl/multiple-active-statements.html>.
+
 In order of preference, they are:
 
 =over 8
 
-=item * L</connect_call_use_mars>
+=item * L<mars|/connect_call_use_mars>
 
-=item * L</connect_call_use_dynamic_cursors>
+=item * L<dynamic_cursors|/connect_call_use_dynamic_cursors>
 
-=item * L</connect_call_use_server_cursors>
+=item * L<server_cursors|/connect_call_use_server_cursors>
 
 =back
 
@@ -95,6 +103,15 @@ Use as:
 
   on_connect_call => 'use_mars'
 
+in your connection info, or alternatively specify it directly:
+
+  Your::Schema->connect (
+    $original_dsn . '; MARS_Connection=Yes',
+    $user,
+    $pass,
+    \%attrs,
+  )
+
 Use to enable a feature of SQL Server 2005 and later, "Multiple Active Result
 Sets". See L<DBD::ODBC::FAQ/Does DBD::ODBC support Multiple Active Statements?>
 for more information.
@@ -115,7 +132,7 @@ sub connect_call_use_mars {
   }
 
   if ($dsn !~ /MARS_Connection=/) {
-    if ($self->using_freetds) {
+    if ($self->_using_freetds) {
       $self->throw_exception('FreeTDS does not support MARS at the time of '
                             .'writing.');
     }
@@ -126,8 +143,9 @@ sub connect_call_use_mars {
     }
 
     if (my ($data_source) = $dsn =~ /^dbi:ODBC:([\w-]+)\z/i) { # prefix with DSN
-      warn "Bare DSN in ODBC connect string, rewriting to DSN=$data_source\n";
-      $dsn = "dbi:ODBC:DSN=$data_source";
+      warn "Bare DSN in ODBC connect string, rewriting as 'dsn=$data_source'"
+          ." for MARS\n";
+      $dsn = "dbi:ODBC:dsn=$data_source";
     }
 
     $self->_dbi_connect_info->[0] = "$dsn;MARS_Connection=Yes";
@@ -148,83 +166,113 @@ Use as:
 
   on_connect_call => 'use_dynamic_cursors'
 
-in your L<connect_info|DBIx::Class::Storage::DBI/connect_info> as one way to enable multiple
-concurrent statements.
+Which will add C<< odbc_cursortype => 2 >> to your DBI connection
+attributes, or alternatively specify the necessary flag directly:
 
-Will add C<< odbc_cursortype => 2 >> to your DBI connection attributes. See
-L<DBD::ODBC/odbc_cursortype> for more information.
+  Your::Schema->connect (@dsn, { ... odbc_cursortype => 2 })
 
-Alternatively, you can add it yourself and dynamic cursor support will be
-automatically enabled.
+See L<DBD::ODBC/odbc_cursortype> for more information.
 
 If you're using FreeTDS, C<tds_version> must be set to at least C<8.0>.
 
 This will not work with CODE ref connect_info's.
 
-B<WARNING:> this will break C<SCOPE_IDENTITY()>, and C<SELECT @@IDENTITY> will
-be used instead, which on SQL Server 2005 and later will return erroneous
-results on tables which have an on insert trigger that inserts into another
-table with an C<IDENTITY> column.
+B<WARNING:> on FreeTDS (and maybe some other drivers) this will break
+C<SCOPE_IDENTITY()>, and C<SELECT @@IDENTITY> will be used instead, which on SQL
+Server 2005 and later will return erroneous results on tables which have an on
+insert trigger that inserts into another table with an C<IDENTITY> column.
+
+B<WARNING:> on FreeTDS, changes made in one statement (e.g. an insert) may not
+be visible from a following statement (e.g. a select.)
+
+B<WARNING:> FreeTDS versions > 0.82 seem to have completely broken the ODBC
+protocol. DBIC will not allow dynamic cursor support with such versions to
+protect your data. Please hassle the authors of FreeTDS to act on the bugs that
+make their driver not overly usable with DBD::ODBC.
 
 =cut
 
 sub connect_call_use_dynamic_cursors {
   my $self = shift;
 
-  if (ref($self->_dbi_connect_info->[0]) eq 'CODE') {
-    $self->throw_exception ('Cannot set DBI attributes on a CODE ref connect_info');
-  }
+  if (($self->_dbic_connect_attributes->{odbc_cursortype} || 0) < 2) {
 
-  my $dbi_attrs = $self->_dbi_connect_info->[-1];
+    my $dbi_inf = $self->_dbi_connect_info;
 
-  unless (ref $dbi_attrs eq 'HASH') {
-    $dbi_attrs = {};
-    push @{ $self->_dbi_connect_info }, $dbi_attrs;
-  }
+    $self->throw_exception ('Cannot set DBI attributes on a CODE ref connect_info')
+      if ref($dbi_inf->[0]) eq 'CODE';
+
+    # reenter connection information with the attribute re-set
+    $dbi_inf->[3] = {} if @$dbi_inf <= 3;
+    $dbi_inf->[3]{odbc_cursortype} = 2;
+
+    $self->_dbi_connect_info($dbi_inf);
 
-  if (not exists $dbi_attrs->{odbc_cursortype}) {
-    # turn on support for multiple concurrent statements, unless overridden
-    $dbi_attrs->{odbc_cursortype} = 2;
     $self->disconnect; # resetting dbi attrs, so have to reconnect
     $self->ensure_connected;
-    $self->_set_dynamic_cursors;
   }
 }
 
-sub _set_dynamic_cursors {
-  my $self = shift;
-  my $dbh  = $self->_get_dbh;
-
-  try {
-    local $dbh->{RaiseError} = 1;
-    local $dbh->{PrintError} = 0;
-    $dbh->do('SELECT @@IDENTITY');
-  } catch {
-    $self->throw_exception (<<'EOF');
-
-Your drivers do not seem to support dynamic cursors (odbc_cursortype => 2),
-if you're using FreeTDS, make sure to set tds_version to 8.0 or greater.
-EOF
-  };
-
-  $self->_using_dynamic_cursors(1);
-  $self->_identity_method('@@identity');
-}
-
-sub _init {
+sub _run_connection_actions {
   my $self = shift;
 
+  $self->next::method (@_);
+
+  # keep the dynamic_cursors_support and driver-state in sync
+  # on every reconnect
+  my $use_dyncursors = ($self->_dbic_connect_attributes->{odbc_cursortype} || 0) > 1;
   if (
-    ref($self->_dbi_connect_info->[0]) ne 'CODE'
-      &&
-    ref ($self->_dbi_connect_info->[-1]) eq 'HASH'
-      &&
-    ($self->_dbi_connect_info->[-1]{odbc_cursortype} || 0) > 1
+    $use_dyncursors
+      xor
+    !!$self->_using_dynamic_cursors
   ) {
-    $self->_set_dynamic_cursors;
+    if ($use_dyncursors) {
+      try {
+        my $dbh = $self->_dbh;
+        local $dbh->{RaiseError} = 1;
+        local $dbh->{PrintError} = 0;
+        $dbh->do('SELECT @@IDENTITY');
+      } catch {
+        $self->throw_exception (
+          'Your drivers do not seem to support dynamic cursors (odbc_cursortype => 2).'
+         . (
+          $self->_using_freetds
+            ? ' If you are using FreeTDS, make sure to set tds_version to 8.0 or greater.'
+            : ''
+          )
+        );
+      };
+
+      $self->_using_dynamic_cursors(1);
+      $self->_identity_method('@@identity');
+    }
+    else {
+      $self->_using_dynamic_cursors(0);
+      $self->_identity_method(undef);
+    }
   }
-  else {
-    $self->_using_dynamic_cursors(0);
+
+  $self->_no_scope_identity_query($self->_using_dynamic_cursors
+    ? $self->_using_freetds
+    : undef
+  );
+
+  # freetds is too damn broken, some fixups
+  if ($self->_using_freetds) {
+
+    # no dynamic cursors starting from 0.83
+    if ($self->_using_dynamic_cursors) {
+      my $fv = $self->_using_freetds_version || 999;  # assume large if can't be determined
+      $self->throw_exception(
+        'Dynamic cursors (odbc_cursortype => 2) are not supported with FreeTDS > 0.82 '
+      . "(you have $fv). Please hassle FreeTDS authors to fix the outstanding bugs in "
+      . 'their driver.'
+      ) if $fv > 0.82
+    }
+
+    # FreeTDS is too broken wrt execute_for_fetch batching
+    # just disable it outright until things quiet down
+    $self->_disable_odbc_array_ops;
   }
 }
 
@@ -260,37 +308,15 @@ sub connect_call_use_server_cursors {
   $self->_get_dbh->{odbc_SQL_ROWSET_SIZE} = $sql_rowset_size;
 }
 
-=head2 using_freetds
-
-Tries to determine, to the best of our ability, whether or not you are using the
-FreeTDS driver with L<DBD::ODBC>.
-
-=cut
-
-sub using_freetds {
-  my $self = shift;
-
-  my $dsn = $self->_dbi_connect_info->[0];
-
-  $dsn = '' if ref $dsn eq 'CODE';
-
-  my $dbh = $self->_get_dbh;
-
-  return 1 if $dsn =~ /driver=FreeTDS/i
-              || (try { $dbh->get_info(6) }||'') =~ /tdsodbc/i;
-
-  return 0;
-}
-
 1;
 
 =head1 AUTHOR
 
-See L<DBIx::Class/CONTRIBUTORS>.
+See L<DBIx::Class/AUTHOR> and L<DBIx::Class/CONTRIBUTORS>.
 
 =head1 LICENSE
 
 You may distribute this code under the same terms as Perl itself.
 
 =cut
-# vim: sw=2 sts=2
+# vim:sw=2 sts=2 et
