@@ -1,6 +1,6 @@
 my ($optdep_msg, $opt_testdeps);
 
-unless ($args->{with_optdeps}) {
+if ($args->{skip_author_deps}) {
   $optdep_msg = <<'EOW';
 
 ******************************************************************************
@@ -9,12 +9,8 @@ unless ($args->{with_optdeps}) {
 *** IGNORING AUTHOR MODE: no optional test dependencies will be forced.    ***
 ***                                                                        ***
 *** If you are using this checkout with the intention of submitting a DBIC ***
-*** patch you may want to aim at running more tests by re-configuring via: ***
-***                                                                        ***
-***  perl Makefile.PL --with-optdeps                                       ***
-***                                                                        ***
-*** which will install all optional dependencies. This is not a mandatory  ***
-*** step - the extensive CI setup will likely catch your mistakes anyway.  ***
+*** patch, you are *STRONGLY ENCOURAGED* to install all dependencies, so   ***
+*** that every possible unit-test will run.                                ***
 ***                                                                        ***
 ******************************************************************************
 ******************************************************************************
@@ -27,8 +23,8 @@ else {
 ******************************************************************************
 ******************************************************************************
 ***                                                                        ***
-*** --with-optdeps specified: converting all optional test dependencies to ***
-*** hard requires ( to disable re-run Makefile.PL without options )        ***
+*** AUTHOR MODE: all optional test dependencies converted to hard requires ***
+***       ( to disable re-run Makefile.PL with --skip-author-deps )        ***
 ***                                                                        ***
 ******************************************************************************
 ******************************************************************************
@@ -36,19 +32,12 @@ else {
 EOW
 
   require DBIx::Class::Optional::Dependencies;
+  my %reqs_for_group = %{DBIx::Class::Optional::Dependencies->req_group_list};
 
   # exclude the rdbms_* groups which are for DBIC users
-  # and the moose-related stuff iff we are under 5.8.3
-  $opt_testdeps = DBIx::Class::Optional::Dependencies->req_list_for([
-    grep {
-      !/^rdbms_|^dist_/
-        and
-      ( "$]" > 5.008002 or !/^ (?: test_ )? (?: admin | admin_script | replicated ) $/x )
-    } keys %{DBIx::Class::Optional::Dependencies->req_group_list}
-  ]);
-
-  # this one is "special" - we need it both in optdeps and as a hard dep
-  delete $opt_testdeps->{'DBD::SQLite'};
+  $opt_testdeps = {
+    map { %{$reqs_for_group{$_}} } grep { !/^rdbms_/ } keys %reqs_for_group
+  };
 
   print "Including all optional deps\n";
   $reqs->{test_requires} = {
@@ -77,9 +66,6 @@ EOW
 # this will run after the Makefile is written and the main Makefile.PL terminates
 #
 END {
-  # shit already hit the fan
-  return if $?;
-
   # Re-write META.yml at the end to _exclude_ all forced build-requires (we do not
   # want to ship this) We are also not using M::I::AuthorRequires as this will be
   # an extra dep, and deps in Makefile.PL still suck
@@ -102,31 +88,16 @@ END {
 
   if (keys %removed_build_requires) {
     print "Regenerating META with author requires excluded\n";
-    # M::I understands unicode in meta but does not write with the right
-    # layers - fhtagn!!!
-    local $SIG{__WARN__} = sub { warn $_[0] unless $_[0] =~ /Wide character in print/ };
     Meta->write;
   }
 
-  # strip possible crlf from META
-  if ($^O eq 'MSWin32' or $^O eq 'cygwin') {
-    local $ENV{PERLIO} = 'unix';
-    system( $^X, qw( -MExtUtils::Command -e dos2unix -- META.yml),  );
-  }
-
   # test that we really took things away (just in case, happened twice somehow)
   if (! -f 'META.yml') {
     warn "No META.yml generated?! aborting...\n";
     unlink 'Makefile';
     exit 1;
   }
-  my $meta = do { local (@ARGV, $/) = 'META.yml'; <> };
-
-  $meta =~ /^\Qname: DBIx-Class\E$/m or do {
-    warn "Seemingly malformed META.yml...?\n";
-    unlink 'Makefile';
-    exit 1;
-  };
+  my $meta = do { local @ARGV = 'META.yml'; local $/; <> };
 
   # this is safe as there is a fatal check earlier in the main Makefile.PL
   # to make sure there are no duplicates (i.e. $opt_testdeps does not contain
