@@ -1,6 +1,3 @@
-# When a long-standing branch is updated a README may still inger around
-unlink 'README' if -f 'README';
-
 # Makefile syntax allows adding extra dep-specs for already-existing targets,
 # and simply appends them on *LAST*-come *FIRST*-serve basis.
 # This allows us to inject extra depenencies for standard EUMM targets
