@@ -1,4 +1,4 @@
-package # hide from PAUSE 
+package # hide from PAUSE
     DBICTest::Schema::CD;
 
 use base qw/DBICTest::BaseResult/;
@@ -23,7 +23,7 @@ __PACKAGE__->add_columns(
     data_type => 'varchar',
     size      => 100,
   },
-  'genreid' => { 
+  'genreid' => {
     data_type => 'integer',
     is_nullable => 1,
     accessor => undef,
@@ -37,17 +37,17 @@ __PACKAGE__->add_columns(
 __PACKAGE__->set_primary_key('cdid');
 __PACKAGE__->add_unique_constraint([ qw/artist title/ ]);
 
-__PACKAGE__->belongs_to( artist => 'DBICTest::Schema::Artist', undef, { 
-    is_deferrable => 1, 
+__PACKAGE__->belongs_to( artist => 'DBICTest::Schema::Artist', undef, {
+    is_deferrable => 1,
     proxy => { artist_name => 'name' },
 });
-__PACKAGE__->belongs_to( very_long_artist_relationship => 'DBICTest::Schema::Artist', 'artist', { 
-    is_deferrable => 1, 
+__PACKAGE__->belongs_to( very_long_artist_relationship => 'DBICTest::Schema::Artist', 'artist', {
+    is_deferrable => 1,
 });
 
 # in case this is a single-cd it promotes a track from another cd
-__PACKAGE__->belongs_to( single_track => 'DBICTest::Schema::Track', 'single_track', 
-    { join_type => 'left'} 
+__PACKAGE__->belongs_to( single_track => 'DBICTest::Schema::Track', 'single_track',
+    { join_type => 'left'}
 );
 
 __PACKAGE__->has_many( tracks => 'DBICTest::Schema::Track' );
@@ -93,4 +93,37 @@ __PACKAGE__->belongs_to('genre_inefficient', 'DBICTest::Schema::Genre',
     },
 );
 
+
+# This is insane. Don't ever do anything like that
+# This is for testing purposes only!
+
+# mst: mo: DBIC is an "object relational mapper"
+# mst: mo: not an "object relational hider-because-mo-doesn't-understand-databases
+# ribasushi: mo: try it with a subselect nevertheless, I'd love to be proven wrong
+# ribasushi: mo: does sqlite actually take this?
+# ribasushi: an order in a correlated subquery is insane - how long does it take you on real data?
+
+__PACKAGE__->might_have(
+    'last_track',
+    'DBICTest::Schema::Track',
+    sub {
+        my $args = shift;
+        return (
+            {
+                "$args->{foreign_alias}.trackid" => { '=' =>
+                    $args->{self_resultsource}->schema->resultset('Track')->search(
+                       { 'correlated_tracks.cd' => { -ident => "$args->{self_alias}.cdid" } },
+                       {
+                          order_by => { -desc => 'position' },
+                          rows     => 1,
+                          alias    => 'correlated_tracks',
+                          columns  => ['trackid']
+                       },
+                    )->as_query
+                }
+            }
+        );
+    },
+);
+
 1;
